@@ -28,10 +28,24 @@
 .onAttach <- function(lib, pkg) {
   ## We start the worker after attaching the library so that
   ## lazy-loaded R code in SPRINT is available to the slave processes
-  invisible(.C("worker"))
+  invisible(return_val <- .Call("worker"))
+
+  ## If the return value is not 0, which indicates a 'worker' MPI
+  ## process, then immediately quit. This will prevent any messages
+  ## printed out, or any other unwanted behavior.
+  if(return_val != 0) {
+    q(save = "no")
+  }
+
   ver <- read.dcf(file.path(lib, pkg, "DESCRIPTION"), "Version")
   ver <- as.character(ver)
   packageStartupMessage("SPRINT ", ver, " loaded\n")
+  packageStartupMessage(paste("Welcome to SPRINT\n",
+                              "Please help us fund SPRINT by filling in \n",
+                              "the form at http://www.r-sprint.org/ \n",
+                              "or emailing us at sprint@ed.ac.uk and letting \n",
+                              "us know whether you use SPRINT for commercial \n",
+                              "or academic use."))
 }
 
 ## Called when the extension is unloaded. This is expected to happen
